// Copyright (c) FIRST and other WPILib contributors.
// Open Source Software; you can modify and/or share it under the terms of
// the WPILib BSD license file in the root directory of this project.

package frc.robot;

import edu.wpi.first.math.geometry.Pose2d;
import edu.wpi.first.wpilibj.shuffleboard.BuiltInWidgets;
import edu.wpi.first.wpilibj.shuffleboard.Shuffleboard;
import edu.wpi.first.wpilibj.shuffleboard.ShuffleboardTab;
import edu.wpi.first.wpilibj2.command.Command;
import edu.wpi.first.wpilibj2.command.Commands;
import edu.wpi.first.wpilibj2.command.SubsystemBase;
import edu.wpi.first.wpilibj2.command.button.CommandXboxController;
import frc.robot.constants.OIConstants;
import frc.robot.interfaces.IDrivetrain;
import frc.robot.subsystems.DifferentialDrivetrain;
import frc.robot.subsystems.FrisbeeShooterWheels;
import frc.robot.utilities.OIProfileManager;

public class RobotContainer {
  private IDrivetrain drivetrain;
  private FrisbeeShooterWheels frisbeeShooterWheels;

  private CommandXboxController driver;

  private OIProfileManager oiProfiles;

  public RobotContainer() {
    drivetrain = new DifferentialDrivetrain(
      new Pose2d(), 
      true, 
      false
    );

    frisbeeShooterWheels = new FrisbeeShooterWheels();

    driver = new CommandXboxController(OIConstants.kDriverUSB);

    oiProfiles = new OIProfileManager(
      (SubsystemBase) drivetrain,
      frisbeeShooterWheels
    );

    oiProfiles.addProfile("Brad's Profile", this::bradProfile);
    oiProfiles.addProfile("Noah's Profile", this::noahProfile);
  }

  public void configureShuffleboard() {
    ShuffleboardTab operatorInterfaceTab = Shuffleboard.getTab(OIConstants.kOITabName);

    operatorInterfaceTab.add("Profile Selector", oiProfiles)
      .withPosition(0, 0)
      .withSize(2, 1)
      .withWidget(BuiltInWidgets.kComboBoxChooser);
  }

  public Command getAutonomousCommand() {
    return Commands.print("No autonomous command configured");
  }

  private void drivetrainConfig() {
    if (drivetrain instanceof DifferentialDrivetrain) {
      ((DifferentialDrivetrain) drivetrain).setDefaultCommand(
        drivetrain.drive(driver)
      );
    }
  }

  private void bradProfile() {
    drivetrainConfig();

    frisbeeShooterWheels.setDefaultCommand(
      frisbeeShooterWheels.setSpeed(driver::getRightTriggerAxis)
    );

    driver.a().whileTrue(
      frisbeeShooterWheels.setSpeed(() -> -.5)
    );
  }

  private void noahProfile() {
    drivetrainConfig();

    frisbeeShooterWheels.setDefaultCommand(
      frisbeeShooterWheels.setSpeed(driver::getLeftTriggerAxis)
    );

    driver.b().whileTrue(
      frisbeeShooterWheels.setSpeed(() -> -.5)
    );
  }
}
